#ifndef SDLSOUND_H
#define SDLSOUND_H

#include <SDL/SDL.h>

#define FILELEN  128
#define MAXSOUNDS 16

struct wavespec
{
	SDL_AudioSpec spec;
	Uint8 *buf;
	Uint32 len;
	Uint32 pos;
};

class SDLWavePlayer
{
    wavespec waves[MAXSOUNDS];
    int numsounds;
    int playing;

public:
    SDLWavePlayer();
    ~SDLWavePlayer();

    int LoadWAV(const char *filename);
    void Play(int sound);

    friend void callback(void *udata, Uint8 *stream, int len);
};

void callback(void *udata, Uint8 *stream, int len);

SDLWavePlayer::SDLWavePlayer()
{
   numsounds = 0;
   playing = -1;
}

SDLWavePlayer::~SDLWavePlayer()
{
   for(int i=0; i < numsounds; i++)
      SDL_FreeWAV(waves[i].buf);
}

int SDLWavePlayer::LoadWAV(const char *filename)
{
   if(numsounds==MAXSOUNDS)
     return -1;

   if(!SDL_LoadWAV(filename, &waves[numsounds].spec, &waves[numsounds].buf, &waves[numsounds].len))
     return -1;

   waves[numsounds].callback = callback;
   return numsounds++;
}

void SDLWavePlayer::Play(int sound)
{
   if(playing >= 0) {
      return;
   }

   if(sound >= numsounds) {
      return;
   }

   playing = sound;

   waves[playing].pos = 0;

   if(SDL_OpenAudio(&waves[playing].spec, NULL) < 0)
      return;

   SDL_PauseAudio(0);
}

void callback(void *udata, Uint8 *stream, int len)
{

}

#endif // SDLSOUND_H