#ifndef WORLD_H
#define WORLD_H

#include <stdlib.h>
#include <time.h>
#include "graphics.h"
#include "objects.h"

#define PI       3.14159265359

//#define EMPTYMAZE

#define NUMTEX   3
#define MAXOBJ   32
#define OBJFILE  "objects.txt"

#define MAXOBJECTS 32

#define OPENNESS 20
#define TRANSG   (48)
#define TRANSB   (32)
#define TRANSR   (0)
#define ASPECT   3

#define DTOU(x) (((x)*CIRCLE)/360)
#define FPART(x) ((x)-(int)(x))
#define INFINITY (1.0e6f)

typedef unsigned char byte;

struct object_queue_entry {
    int u;
    int v;
    int w;
    float d;
    int t;
};

class World
{
    int width;
    int height;
    byte *world;
    SDL_Rect rec;
    int RefHeight;
    int ObjHeight;

    void BurrowMaze(int x, int y);
    void randopen(int &x, int &y);
    int openthis();
    void RayCast(float x, float y, int angle, int col, int diff_angle);
    inline float abs(float x) { return (x < 0) ? -x : x; }

    SDL_Surface *tex[NUMTEX], *obj[MAXOBJ];
    Object objects[MAXOBJECTS];
    int numobjects;
    object_queue_entry queue[MAXOBJECTS];

public:
    World();
    ~World();

    void set(int x, int y, byte val)
    {
        world[y*width + x] = val;
    }

    byte get(int x, int y)
    {
        return (byte)0x7F & world[y * width + x];
    }

    void setobj(int x, int y)
    {
        world[y * width + x] |= (byte)0x80;
    }

    byte getobj(int x, int y)
    {
         return world[y * width + x];
    }

    void Clear(int tex = 0);
    void Resize(int x, int y);
    void Draw2D(int u, int v, int w, int h);
    void Draw2DTrans(int u, int v, int w, int h);
    void Quick2D(int u, int v, int bw);
    void Quick2DTrans(int u, int v, int bw);
    void CreateMaze(int tex = 1);
    void CreateTitle();
    void Render3D(float x, float y, int angle);
    void RenderObjects();

    void TexRepColor(int tex, int &r, int &g, int &b);
};

#endif // WORLD_H
